#!/usr/bin/python3

"""
At first we define interfaces required for a program
"""
class AbstractAppetizer:
    def eat(self):
        raise NotImplemented

class AbstractMainCourse:
    def eat(self):
        raise NotImplemented

class AbstractDessert:
    def eat(self):
        raise NotImplemented

"""
Also we define our factory interface, factory is a thing that
creates concrete objects implemting interfaces above, different
factories create different objects, but all objects implement the
same interface, and all factories implement the same interface. 
"""
class AbstractDinner:
    def buy_appetizer(self):
        raise NotImplemented

    def buy_main_course(self):
        raise NotImplemented

    def buy_dessert(self):
        raise NotImplemented

"""
Next we implement interfaces required for a program, there may be
many such implementation, for example, we can create implementation
for every OS/hardware architecture/build configuration/etc. Or like
in this example, different kinds of appetizer, main dish and dessert.
"""
class Olivier(AbstractAppetizer):
    def eat(self):
        print('Discusting Olivier')

class Borsch(AbstractAppetizer):
    def eat(self):
        print('Great Borsch')

class FiletMignon(AbstractMainCourse):
    def eat(self):
        print('I like beef!')

class BeefStroganoff(AbstractMainCourse):
    def eat(self):
        print('National Russian dish')

class CremeBrulee(AbstractDessert):
    def eat(self):
        print('Yami-Yami!')

class Blini(AbstractDessert):
    def eat(self):
        print('Neverending classic')

"""
And finally we create different implementations of factory interface.
And again, there may be many such implementation, depending on build
configuration/architecture/OS/etc.
"""
class FrenchDinner(AbstractDinner):
    def buy_appetizer(self):
        return Olivier()

    def buy_main_course(self):
        return FiletMignon()

    def buy_dessert(self):
        return CremeBrulee()

class RussianDinner(AbstractDinner):
    def buy_appetizer(self):
        return Borsch()

    def buy_main_course(self):
        return BeefStroganoff()

    def buy_dessert(self):
        return Blini()

if __name__ == '__main__':
    import locale

    def get_factory():
        factories = {'ru': RussianDinner, 'fr': FrenchDinner}
        lang = locale.getdefaultlocale()[0].lower()[:2]
        return factories.get(lang, RussianDinner)()

    dinner = get_factory()
    dinner.buy_appetizer().eat() 
    dinner.buy_main_course().eat() 
    dinner.buy_dessert().eat() 
